#include <condition_variable>
#include <queue>
#include <iostream>
#include <sstream>
#include <thread>
#include <conio.h>

using namespace std;

mutex m, mq;
condition_variable cv;

queue<int> q;

void introduceRandomDelay(unsigned d)
{
	// simulates a delay between 0 and d seconds.
	this_thread::sleep_for(chrono::seconds(rand()%d));
}

void producer()
{
	stringstream ss;
	for (int i = 0;i<5;++i)
	{
		introduceRandomDelay(1);
		mq.lock();	q.push(i);	mq.unlock();
		ss.str("");
		ss << "Producer: element " << i << " queued." << endl;
		cout << ss.str();
		cv.notify_all();
	}
}

void consumer()
{
	unique_lock<mutex> l(m);
	int failed_attempts = 0;
	stringstream ss;
	while (true)
	{
		mq.lock();
		if (q.size())
		{
			int elem = q.front();
			q.pop();
			mq.unlock();
			failed_attempts = 0;
			ss.str("");
			ss << "Consumer: fetching " << elem << " from queue." << endl;
			cout << ss.str();
			introduceRandomDelay(1);
		} else {
			mq.unlock();
			if (++failed_attempts>1)
			{
				ss.str("");
				ss << "Consumer: two consecutive failed attempts -> Exiting." << endl;
				cout << ss.str();
				break;
			} else {
				ss.str("");
				ss << "Consumer: queue not ready -> going to sleep." << endl;
				cout << ss.str();
				cv.wait_for(l, chrono::seconds(2));
			}
		}
	}
}

int main(int argc, char *argv[])
{
	cout.sync_with_stdio(true);

	thread th(consumer);
	producer();
	th.join();

	cout << "Press any key...";
	_getch();
}