#include <vector>
#include <algorithm>
#include <utility>
#include <iostream>
#include <sstream>
#include <conio.h>
#include <thread>

#ifndef THREAD_LOCAL_CPP11
#define	thread_local __declspec(thread)
#endif

using namespace std;

const unsigned THRESHOLD = 100000;
thread_local unsigned sum_total = 0;

vector<unsigned> v;

void scramble_vector(size_t total_elements)
{
	for (size_t i = 0;i<total_elements;++i)
		v.push_back(i);

	for_each(begin(v), end(v), [](unsigned i) {
		size_t rnd = rand()%v.size();
		swap(v[i], v[rnd]);
	});
}

void sum_until_element_with_threshold(unsigned element, unsigned threshold, exception_ptr& pExc)
{
	try{
		find_if_not(begin(v), end(v), [=](const unsigned i) -> bool {
			bool ret = (i!=element);
			sum_total+= i;
			if (sum_total>threshold)
				throw runtime_error("Sum exceeded threshold.");
			return ret;
		});
		stringstream ss;
		ss << "(Thread #" << this_thread::get_id() << ") " <<
			"Sum of elements until " << element << " is found: " << sum_total << endl;
		cout << ss.str();
	} catch (...) {
		pExc = current_exception();
	}
}

inline void dealWithExceptionIfAny(exception_ptr pExc)
{
	try
	{
		if (!(pExc==exception_ptr()))
			rethrow_exception(pExc);
	} catch (const exception& exc) {
		cout << "(Main thread) Exception received from thread: " << exc.what() << endl;
	}
}

int main(int argc, char *argv[])
{
	cout.sync_with_stdio(true);
	exception_ptr pExc1, pExc2;

	scramble_vector(1000);
	thread th(sum_until_element_with_threshold, 0, THRESHOLD, ref(pExc1));
	sum_until_element_with_threshold(100, THRESHOLD, ref(pExc2));
	th.join();

	dealWithExceptionIfAny(pExc1);
	dealWithExceptionIfAny(pExc2);

	cout << "Press any key...";
	_getch();
}